package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.entities.UserInfo;
import gov.anl.aps.dm.portal.model.beans.UserInfoFacade;
import gov.anl.aps.dm.portal.utilities.SessionUtility;

import java.util.List;
import javax.ejb.EJB;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import javax.inject.Named;
import org.apache.log4j.Logger;

@Named("userInfoController")
@SessionScoped
public class UserInfoController extends CrudEntityController<UserInfo, UserInfoFacade>
{

    private static final Logger logger = Logger.getLogger(UserInfoController.class.getName());

    @EJB
    private UserInfoFacade userInfoFacade;

    private String passwordEntry = null;
    
    public UserInfoController() {
    }

    @Override
    protected UserInfoFacade getFacade() {
        return userInfoFacade;
    }

    @Override
    protected UserInfo createEntityInstance() {
        return new UserInfo();
    }

    @Override
    public String getEntityTypeName() {
        return "userInfo";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getUsername();
        }
        return "";
    }

    @Override
    public List<UserInfo> getAvailableItems() {
        return super.getAvailableItems();
    }

    @Override
    public String prepareEdit(UserInfo user) {
        passwordEntry = null;
        return super.prepareEdit(user);
    }

    @Override
    public void prepareEntityInsert(UserInfo user) throws ObjectAlreadyExists {
        UserInfo existingUser = userInfoFacade.findByUsername(user.getUsername());
        if (existingUser != null) {
            throw new ObjectAlreadyExists("User " + user.getUsername() + " already exists.");
        }
        logger.debug("Inserting new user " + user.getUsername());
    }

    @Override
    public void prepareEntityUpdate(UserInfo user) throws DmPortalException {
        if (passwordEntry != null && !passwordEntry.isEmpty()) {
            user.setPassword(passwordEntry);
        }
        passwordEntry = null;
    }

     @Override
     protected String getObjectAlreadyExistMessage(UserInfo user)  {
        if (user == null)
            return null;
        return "User " + user.getUsername() + " already exists."; 
    }


    public String prepareSessionUserEdit(String viewPath) {
        UserInfo sessionUser = (UserInfo) SessionUtility.getUser();
        if (sessionUser == null) {
            return null;
        }
        prepareEdit(sessionUser);
        return viewPath + "?faces-redirect=true";
    }
    
    public String getPasswordEntry() {
        return passwordEntry;
    }

    public void setPasswordEntry(String passwordEntry) {
        this.passwordEntry = passwordEntry;
    }

@FacesConverter(forClass = UserInfo.class)
    public static class UserInfoControllerConverter implements Converter
    {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            UserInfoController controller = (UserInfoController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "userInfoController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof UserInfo) {
                UserInfo o = (UserInfo) object;
                return getStringKey(o.getId());
            }
            else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + UserInfo.class.getName());
            }
        }

    }

}